// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "PlayerController.hpp"
#include "game/DawnGame.hpp"

#include "scene/components/physics/3d/CapsuleCollider.hpp"

using namespace Dawn;

PlayerController::PlayerController(SceneItem *item) : SceneItemComponent(item) {

}

void PlayerController::onStart() {
  useEvent([&](float_t delta){
    auto camera = getScene()->findComponent<Camera>();
    assertNotNull(camera);

    // Friction
    velocity -= velocity * friction * delta;

    // Movement
    auto inputMove = getGame()->inputManager.getAxis2D(
      INPUT_BIND_NEGATIVE_X, INPUT_BIND_POSITIVE_X,
      INPUT_BIND_NEGATIVE_Y, INPUT_BIND_POSITIVE_Y
    ) * delta * moveSpeed;
    
    if(inputMove.x != 0 || inputMove.y != 0) {
      float_t angle = atan2(inputMove.y, inputMove.x);
      glm::vec3 movement = glm::vec3(cos(angle), 0, sin(angle));
      velocity += movement * delta * moveSpeed;
    }

    // Move / Update
    transform->setLocalPosition(transform->getLocalPosition() + (velocity * delta));


    // tEST
    auto collider = item->getComponent<CapsuleCollider>();
    Collider3DRayResult result;

    struct Ray3D ray;
    ray.origin = glm::vec3(0, 0.5f, 1);
    ray.direction = glm::vec3(10, 0, 0);

    struct Color color = COLOR_MAGENTA;

    if(collider->raycast(&result, ray)) {
      // std::cout << "Collide!" << std::endl;
      color = COLOR_WHITE;

      getScene()->debugRay({
        .start = result.point,
        .direction = (result.normal * 100.0f),
        .color = COLOR_GREEN
      });
    }

    getScene()->debugRay({
      .start = ray.origin,
      .direction = ray.direction,
      .color = color
    });

  }, getScene()->eventSceneUpdate);
}