// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "scene/Scene.hpp"
#include "scene/components/PlayerController.hpp"
#include "scene/components/GameCamera.hpp"
#include "scene/components/physics/3d/CubeCollider.hpp"
#include "scene/components/physics/3d/CapsuleCollider.hpp"
#include "scene/components/GameCamera.hpp"
#include "scene/components/display/MeshRenderer.hpp"
#include "scene/components/display/MeshHost.hpp"
#include "scene/components/display/material/SimpleTexturedMaterial.hpp"
#include "display/mesh/CapsuleMesh.hpp"
#include "scene/components/example/ExampleSpin.hpp"

namespace Dawn {
  class HelloWorldScene : public Scene {
    protected:
      Camera *camera;

      void stage() override {

        auto playerItem = this->createSceneItem();
        auto player = playerItem->addComponent<PlayerController>();
        auto hitbox = playerItem->addComponent<CapsuleCollider>();
        playerItem->addComponent<MeshRenderer>();
        auto meshHost = playerItem->addComponent<MeshHost>();
        playerItem->addComponent<SimpleTexturedMaterial>();

        CapsuleMesh::create(&meshHost->mesh, hitbox->radius, hitbox->height);

        auto wall = this->createSceneItem();
        auto wallHitbox = wall->addComponent<CubeCollider>();
        wall->transform.setLocalPosition(glm::vec3(-10, 0, 0));
        wallHitbox->min = -(wallHitbox->max = glm::vec3(1, 1, 5));

        camera = Camera::create(this);
        camera->transform->lookAt(glm::vec3(4, 4, 4), glm::vec3(0, 0, 0));
        // auto gameCamera = camera->item->addComponent<GameCamera>();
        // gameCamera->player = player;
      }
      
      std::vector<Asset*> getRequiredAssets() override {
        auto assMan = &this->game->assetManager;
        std::vector<Asset*> assets;
        return assets;
      }

    public:
      HelloWorldScene(DawnGame *game) : Scene(game) {}
  };
}